/* Shoot or Die! - Game.cpp
 * Copyright (c) 2008, 2009 by Michael Borgmann <mail@lechuck.de>
 * This code is released under the ISC license
 */

#include <iostream>
#include <sstream>
#include "OpenGLDriver.h"
#include "OpenGLImage.h"
#include "OpenGLGraphics.h"
#include "OpenGLSprite.h"
#include "OpenGLTilemap.h"
#include "SDLDriver.h"
#include "SDLGraphics.h"
#include "SDLImage.h"
#include "SDLSprite.h"
#include "SDLTilemap.h"
#include "Starfield.h"
#include "Game.h"

Game::Game(std::string t, std::string r) : TITLE(t), RELEASE(r) {

	/* initialization */

	// screen resolution
	//width = 800; height = 600; bpp = 32;		// optional 960x600x32
	width = 1680; height = 1050; bpp = 32;		// optional 960x600x32
	driver = SDL;
	driver = OPENGL;

	// command line output
	std::cout << TITLE << " " << RELEASE << std::endl;

	// set up SDL
	if (SDL_Init(SDL_INIT_EVERYTHING == -1)) {
		std::cerr << "Unable to initialize SDL: " << SDL_GetError() << std::endl;
		exit(1);
	}

	atexit(SDL_Quit);

	/* initialize SDL subsystems */

	// setup audio system (SDL_Mixer)
	if (Mix_OpenAudio(22050, MIX_DEFAULT_FORMAT, 2, 2048) == -1) {
		std::cerr << "Unable to initialize the audio system: " << Mix_GetError() << std::endl;
		exit(1);
	}

	// setup fonts rendering (SDL_TTF)
	if (TTF_Init() == -1) {
		std::cerr << "Unable to initialize TTF: " << SDL_GetError() << std::endl;
		exit(1);
	}

	/* initialize video driver */
	if (driver == OPENGL) {
		video = new OpenGLDriver(width, height, bpp, TITLE, true);
		screen = new OpenGLGraphics();
	}
	else if (driver == SDL) {
		video = new SDLDriver(width, height, bpp, TITLE, true);
		screen = new SDLGraphics(SDL_GetVideoSurface());
	}
	video->ShowMouse(false);

	// initialize files
	Create();
	Demo();
}

Game::~Game() {

	// free files
	Destroy();

	// close systems
	Mix_CloseAudio();
	TTF_Quit();
	SDL_Quit();
}

bool Game::Create() {

	// create driver objects
	if (driver == OPENGL) {				// OPENGL
		background = new OpenGLImage();
		turricana = new OpenGLImage();
		player = new OpenGLSprite("share/Footywiz.png", 0, 4);
		map = new OpenGLTilemap("share/test.map");
	}
	else if (driver == SDL) {			// SDL
		background = new SDLImage();
		turricana = new SDLImage();
		player = new SDLSprite("share/Footywiz.png", 0, 4);
		map = new SDLTilemap("share/test.map");
	}

	// load images
	background->LoadImage("share/Shoot.or.Die!.png", 0);
	turricana->LoadImage("share/Biatch.png", 0xff00ff);

	// load sprites	
	player->SetPosition(width/2-16, height/2-16);

	// load font
	font = TTF_OpenFont("share/formplex12.ttf", 28);
	if (!font) {
		std::cerr << "Unable to load TTF font: " << SDL_GetError() << std::endl;
		exit(1);
	}

	// set textcolor
	textcolor.r = 255;
	textcolor.g = 255;
	textcolor.b = 255;

	// load audio files
	sample = NULL;
	music = Mix_LoadMUS("share/Miles_in_the_Sky.ogg");
	if (!music) {
		std::cerr << "Couldn't load music: " << Mix_GetError() << std::endl;
		exit(1);
	}

	return true;
}

bool Game::Destroy() {

	delete screen;
	delete background;
	delete turricana;

	delete player;
	delete map;

	TTF_CloseFont(font);

	Mix_FreeMusic(music);
	Mix_FreeChunk(sample);

	delete video;

	return true;
}

void Game::Demo() {		// NOTE remove for release

	int xvel = 0, yvel = 0;
	int world_x = 0, world_y = 0;
	int camera_width = 8, camera_height = 6;
	int draw_x = width/2 - 80*4, draw_y = 400;

	map->SetCamera(camera_width, camera_height, draw_x, draw_y);

	// play music
	if (Mix_PlayMusic(music, -1) == -1) {
		std::cerr << "Can't play music: " << Mix_GetError() << std::endl;
		exit(1);
	}

	// setup frames per second
	Uint8 frames_per_second = 25;
	int frame = 0;
	int framerate = 0;
	Uint32 frame_timer = SDL_GetTicks();
	Uint32 fps_timer = frame_timer;
	int cap = false;

	message = TTF_RenderText_Solid(font, "FPS: ", textcolor);

	Starfield starfield(screen, width, height, 5000);

	clip.x = 0;
	clip.y = 0;
	clip.width = 32;
	clip.height = 32;

	// main loop
	bool done = false;
	while (!done) {

		// check for input events
		while (SDL_PollEvent(&event)) {
			switch (event.type) {
				case SDL_KEYDOWN:
					switch (event.key.keysym.sym) {
						case SDLK_ESCAPE: done = true;
							break;
						case SDLK_UP:
							if (event.key.keysym.mod & KMOD_SHIFT) player->YVelocity(-10); else player->YVelocity(-5);
							yvel = -2;
							break;
						case SDLK_DOWN:
							player->YVelocity(+5);
							yvel = +2;
							break;
						case SDLK_LEFT:
							player->XVelocity(-5); clip.y = 0; player->Animate(true);
							xvel = -2;
							break;
						case SDLK_RIGHT:
							player->XVelocity(5); clip.y = 32; player->Animate(true);
							xvel = +2;
							break;
						case SDLK_SPACE:
							if (Mix_PlayingMusic()) Mix_HaltMusic();
							else Mix_PlayMusic(music, -1);
							break;
						case SDLK_p:
							if (Mix_PausedMusic() == 1) Mix_ResumeMusic();
							else Mix_PauseMusic();
							break;
						case SDLK_c: cap = !cap; break;
						default:;
					}
					break;
				case SDL_KEYUP:
					switch (event.key.keysym.sym) {
						case SDLK_UP:
							player->YVelocity();
							xvel = 0;
							break;
						case SDLK_DOWN:
							player->YVelocity();
							xvel = 0;
							break;
						case SDLK_LEFT:
							player->XVelocity(); player->Animate(false);
							yvel = 0;
							break;
						case SDLK_RIGHT:
							player->XVelocity(); player->Animate(false);
							yvel = 0;	
							break;
						default:;
					}
					break;
				case SDL_QUIT:
					done = true;
					break;
			}
		}

		// render scene
		video->BeginDraw();
		starfield.Draw();
		screen->DrawImage(background);
		screen->DrawImage(turricana, 1250, 450);
		screen->PutPixel(width/2, height/2, 0xff0000);
		map->Draw(NULL, world_x += xvel, world_y += yvel);
		player->Move();
		player->Draw(NULL, &clip);
		SDLImage* msg = reinterpret_cast<SDLImage*>(message);
		if (cap) screen->Blit(msg, msg);
		video->EndDraw();

		// set frame
		frame++;
		if (frame >= frames_per_second) frame = 0;

		// frame rate regulation
		Uint32 rest = SDL_GetTicks() - frame_timer;
		if (rest < 1000 / frames_per_second) {
			SDL_Delay((1000 / frames_per_second) - rest);
		}
		frame_timer = SDL_GetTicks();

		// prames per second
		if (cap) {
			framerate++;
			if ( fps_timer <= SDL_GetTicks() - 1000) {
				fps_timer = SDL_GetTicks();
				std::stringstream fps;
				fps << "FPS: " << framerate;
				message = TTF_RenderText_Solid(font, fps.str().c_str(), textcolor);
				framerate = 0;
			}
		}
	}
}
