/* Shoot or Die! - Tilemap.cpp
 * Copyright (c) 2008, 2009 by Michael Borgmann <mail@lechuck.de>
 * This code is released under the ISC license
 */

#include <iostream>
#include <fstream>
#include "Tilemap.h"

bool Tilemap::LoadMap(std::string filename) {

	mapfile = filename;

	// open map
	std::ifstream data(mapfile.c_str());
	if (!data) {
		std::cerr << "ERROR: Couldn't load \"" << mapfile << "\".\n";
		data.close();
		return false;
	}

	// get header data
	getline(data, engine);
	data >> version;
	data.seekg(1, std::ios::cur);
	getline(data, map_name);
	getline(data, map_description);
	getline(data, tilesetfile);
	data >> tile_width;
	data >> tile_height;
	data >> map_width;
	data >> map_height;

	// initialize tilemap
	for (int x = 0; x < map_width; x++)
		tiles.push_back(std::vector<int>(map_height));

	// get tilemap
	for (int y = 0; y < map_height; y++)
		for (int x = 0; x < map_width; x++)
			data >> tiles[x][y];

	// close file
	data.close();

	return true;
}

bool Tilemap::LoadTileset(std::string filename) {

	// load tileset
	tileset->LoadImage(filename, 0);

	// get number of x/y tiles
	tiles_x = tileset->Width() / tile_width;
	tiles_y = tileset->Height() / tile_height;
	total_tiles = tiles_x * tiles_y;
	total_width = map_width * tile_width;
	total_height = map_height * tile_height;

	return true;
}

Tilemap::Tilemap(std::string filename) : tile_width(0), tile_height(0), camera_width(0), camera_height(0) {}

void Tilemap::Draw(Image* dest, int world_x, int world_y) {

	// check map boundaries
	if (world_x < 0) world_x = 0;
	if (world_x > max_camera_x) world_x = max_camera_x;
	if (world_y < 0) world_y = 0;
	if (world_y > max_camera_y) world_y = max_camera_y;

	// draw tile map
	for (int y = 0; y < camera_height + 1; y++) {
		for (int x = 0; x < camera_width + 1; x++) {

			// get tile coordinate
			int xtile = world_x / tile_width;
			int ytile = world_y / tile_height;

			// get tile offset (fine scroll)
			int offset_x = world_x % tile_width;		// draw to the left
			int offset_y = world_y % tile_height;		// draw upwards

			// get tile to draw
			clip.x = (tiles[x + xtile][y + ytile] % tiles_x) * tile_width; // (tile_width + 1) + 1;
			clip.y = (tiles[x + xtile][y + ytile] / tiles_x) * tile_height; //(tile_height + 1) + 1;
			clip.width = tile_width;
			clip.height = tile_height;

			// set drawing position
			int draw_x = x * tile_width - offset_x + screen_x;
			int draw_y = y * tile_height - offset_y + screen_y;

			// clip left boarders
			if (draw_x < screen_x) {
				draw_x += offset_x;
				clip.x += offset_x;
				clip.width -= offset_x;
			}

			// clip upper boarders
			if (draw_y < screen_y) {
				draw_y += offset_y;
				clip.y += offset_y;
				clip.height -= offset_y;
			}

			// clip right boarders
			if (draw_x + offset_x - screen_x > tile_width * camera_width - 1) {
				clip.width = offset_x;
			}

			// clip lower boarders
			if (draw_y + offset_y - screen_y > tile_height * camera_height - 1) {
				clip.height = offset_y;
			}

			// blit tile
			graphics->Blit(tileset, dest, &clip, draw_x, draw_y);
		}
	}
}

void Tilemap::SetCamera(int width, int height, int xpos, int ypos) {

	// set camera size in tiles
	camera_width = width;
	camera_height = height;

	// set drawing position
	screen_x = xpos;
	screen_y = ypos;

	// set scrolling bounds
	max_camera_x = total_width - camera_width * tile_width - 1;
	max_camera_y = total_height - camera_height * tile_height - 1;
}
